@@ -1,5 +1,5 @@
 __name__ = "curlylint"
-__version__ = "0.6.0"
+__version__ = "0.8.0"
 __description__ = "{{ 🎀}} Prototype linter for Jinja and Django templates, forked from jinjalint"
 __author__ = "Thibaud Colas"
 __author_email__ = "thibaudcolas@gmail.com"
@@ -8,6 +8,6 @@ __license__ = "MIT"
 __copyright__ = "Copyright 2020-present Thibaud Colas"
 
 if __name__ == "__main__":
-    from .cli import main
+    from .cli import patched_main
 
-    main()
+    patched_main()
